@@ -51,7 +51,7 @@ class HumanReadableTextFormatting:
         return pct
 
     @staticmethod
-    def _longnum2string(x):
+    def _longnum2string(x, min_digits=2):
         if x == 0:
             return '0'
 
@@ -64,7 +64,20 @@ class HumanReadableTextFormatting:
         units = ['', 'K', 'M', 'G', 'T', 'P']
         k = 1000.0
         magnitude = int(floor(log(x, k)))
-        return f'{sign}{round(x / k**magnitude)}{units[magnitude]}'
+        magnitude = np.max((magnitude, 0))
+
+        # first get the rounded value
+        value = round(x / (k**(magnitude)))
+        value_len = len(str(value))
+
+        # if length of rounded value is < min digits
+        # add digits to rounded value
+        if value_len < min_digits and x > k:
+            value = x / (k**(magnitude))
+            n_decimals = min_digits - value_len
+            value = round(value*(10**n_decimals))/(10**n_decimals)
+
+        return f'{sign}{value}{units[magnitude]}'
 
     @staticmethod
     def html_text_augment(x, methods):
@@ -248,23 +261,105 @@ def update_yaxis_from_dict(fig, trace_dict):
                                  for t in vals['trace'] if hasattr(t, 'y')])
         ymin, ymax, step = get_new_axis_range(ax_data)
         # exception for very small range values
-        if step > 0:
-            tick_steps = np.arange(ymin, ymax, step)
-            tick_steps = downsample_steps(tick_steps, ymax)
-
-            # probably no need to cap ymax, just keep it at original calculated ymax
-            # ymax=cap_ymax(ymax, tick_steps, fig_data.y)
-            print('yaxis details:')
-            print(ymin, ymax)
-            print(tick_steps)
 
+        tick_steps = np.arange(ymin, ymax, step)
+        if ymin < 0 and ymax > 0:
+            # shift to include origin if negative values in range
             fig.update_yaxes(
-                selector=vals['selector'],
-                tickmode='array',
-
-                tickvals=tick_steps[1:],
-                range=[ymin, ymax],
+                zeroline=True,
+                zerolinewidth=1.1,
+                zerolinecolor='#C8CDD5'
             )
+            tick_steps = shift_steps_include_origin(tick_steps)
+
+        # down sample last step, dont re adjust min max, just take fewer ticks
+        tick_steps = downsample_steps(tick_steps)
+        print('yaxis details:')
+        print(ymin, ymax)
+        print(tick_steps)
+
+        fig.update_yaxes(
+            selector=vals['selector'],
+            tickmode='array',
+
+            tickvals=tick_steps[1:],
+            range=[ymin, ymax],
+        )
+
+
+def get_new_axis_range(fig_data):
+
+    ymin = np.min(fig_data)
+    ymax = np.max(fig_data)
+
+    ymin, ymax, rng = rescale_min_max_for_data_labels(ymin, ymax)
+
+    step = rng/4
+    print('step', step)
+    step = adjust_step(step)
+    print('adjusted_step', step)
+
+    # min up or max down, depending on axis range
+    if ymin > 0:
+        ymin = round_x_to_nearest_y(ymin, step, method='floor')
+
+    if ymax < 0:
+        ymax = round_x_to_nearest_y(ymax, step, method='ceil')
+
+    # ymax=round_x_to_nearest_y(ymax, step, method = 'ceil')
+    return ymin, ymax, step
+
+
+def rescale_min_max_for_data_labels(ymin, ymax):
+    rng = ymax-ymin
+    print('before scaling', ymin, ymax)
+    # rescale to add buffer for data labels
+    if ymin >= 0 and ymax > 0:
+        ymax += rng * .15
+    if ymin < 0 and ymax <= 0:
+        ymin -= rng + .05
+        ymax += rng + .1
+    if ymin < 0 and ymax > 0:
+        ymin -= rng * .125
+        ymax += rng * .125
+
+    rng = ymax-ymin
+    print('after scaling', ymin, ymax)
+    return ymin, ymax, rng
+
+
+def adjust_step(step):
+
+    modifier = 1
+    return np.power(10, np.floor(np.log10(step))) * modifier
+
+
+def round_x_to_nearest_y(x, y, method='ceil'):
+    if method == 'ceil':
+        return np.ceil(x/y)*y
+    elif method == 'floor':
+        return np.floor(x/y)*y
+
+
+def shift_steps_include_origin(tick_steps):
+    # if more ticks greater than zero, shift down
+    # else shift up
+    if (tick_steps > 0).sum()/len(tick_steps) >= 0.5:
+        shift = tick_steps[tick_steps > 0].min()
+    else:
+        shift = tick_steps[tick_steps < 0].max()
+
+    tick_steps -= shift
+    return tick_steps
+
+
+def downsample_steps(tick_steps, max_ticks=6):
+    n_ticks = len(tick_steps)
+    if n_ticks > max_ticks:
+        sample_factor = int(np.ceil(n_ticks / max_ticks))
+        tick_steps = np.array([tick_steps[idx]
+                               for idx in range(0, n_ticks, sample_factor)])
+    return tick_steps
 
 
 def assign_colors_from_dict(trace_dict, theme):
@@ -319,17 +414,6 @@ def update_bar_width_from_dict(trace_dict):
             trace.update({'width': bar_width})
 
 
-def downsample_steps(tick_steps, max_val, max_ticks=6):
-    n_ticks = len(tick_steps)
-    if n_ticks > max_ticks:
-        sample_factor = int(np.ceil(n_ticks / max_ticks))
-        tick_steps = np.array([tick_steps[idx]
-                               for idx in range(0, n_ticks, sample_factor)])
-
-    # max_tick_idx = np.where(tick_steps > max_val)[0][0] + 1
-    return tick_steps
-
-
 def cap_ymax(ymax, tick_steps, ydata):
     if tick_steps[-1] > np.max(ydata):
         ymax = tick_steps[-1]
@@ -337,34 +421,6 @@ def cap_ymax(ymax, tick_steps, ydata):
     return ymax
 
 
-def get_new_axis_range(fig_data):
-    ymin = np.min(fig_data) * .9
-    ymax = np.max(fig_data) * 1.1
-    rng = ymax-ymin
-
-    step = rng/4
-    step = adjust_step(step)
-
-    # round ymin only, to set starting point for tick steps
-    # ymax not necessary
-    ymin = round_x_to_nearest_y(ymin, step, method='floor')
-    # ymax=round_x_to_nearest_y(ymax, step, method = 'ceil')
-
-    return ymin, ymax, step
-
-
-def adjust_step(step):
-    modifier = 1
-    return np.power(10, np.floor(np.log10(step))) * modifier
-
-
-def round_x_to_nearest_y(x, y, method='ceil'):
-    if method == 'ceil':
-        return np.ceil(x/y)*y
-    elif method == 'floor':
-        return np.floor(x/y)*y
-
-
 #not used
 def update_plot_grid(fig):
     for idx in range(len(fig.data)):
@@ -380,12 +436,20 @@ def add_grid_update_yaxis(fig, idx):
     fig_data = fig.data[idx]
     n_bars = len(fig_data.x)
     ymin, ymax, step = get_new_axis_range(fig_data.y)
-
     # exception for very small range values
     if step > 0:
+        print('doing something')
         tick_steps = np.arange(ymin, ymax, step)
         tick_steps = downsample_steps(tick_steps, ymax)
 
+        # if range is negative, shift to include zero as step
+        print(ymin)
+        print(ymin < 0)
+        if ymin < 0:
+            tick_steps = shift_steps_include_origin(tick_steps)
+            #ymin = tick_steps.min()
+            #ymax = tick_steps.max()
+
         # probably no need to cap ymax, just keep it at original calculated ymax
         # ymax=cap_ymax(ymax, tick_steps, fig_data.y)
         print('yaxis details:')
@@ -413,7 +477,6 @@ def add_grid_update_yaxis(fig, idx):
                           row=idx+1, col=1
                           )
     return fig
-
 # cap the max value if the maximum tick value is already
 # larger than the max y value
 
